@@ -2,7 +2,7 @@
 from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.cpyext.test.test_api import BaseApiTest
 from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
-from pypy.module.cpyext.bytesobject import new_empty_str, PyStringObject
+from pypy.module.cpyext.bytesobject import new_empty_str, PyBytesObject
 from pypy.module.cpyext.api import PyObjectP, PyObject, Py_ssize_tP, generic_cpy_call
 from pypy.module.cpyext.pyobject import Py_DecRef, from_ref, make_ref
 from pypy.module.cpyext.typeobjectdefs import PyTypeObjectPtr
@@ -10,78 +10,61 @@ from pypy.module.cpyext.typeobjectdefs import PyTypeObjectPtr
 import py
 import sys
 
-class AppTestStringObject(AppTestCpythonExtensionBase):
-    def test_stringobject(self):
+class AppTestBytesObject(AppTestCpythonExtensionBase):
+    def test_bytesobject(self):
         module = self.import_extension('foo', [
             ("get_hello1", "METH_NOARGS",
              """
-                 return PyString_FromStringAndSize(
+                 return PyBytes_FromStringAndSize(
                      "Hello world<should not be included>", 11);
              """),
             ("get_hello2", "METH_NOARGS",
              """
-                 return PyString_FromString("Hello world");
+                 return PyBytes_FromString("Hello world");
              """),
             ("test_Size", "METH_NOARGS",
              """
-                 PyObject* s = PyString_FromString("Hello world");
-                 int result = 0;
-                 size_t expected_size;
+                 PyObject* s = PyBytes_FromString("Hello world");
+                 int result = PyBytes_Size(s);
 
-                 if(PyString_Size(s) == 11) {
-                     result = 1;
-                 }
-                 #ifdef PYPY_VERSION
-                    expected_size = sizeof(void*)*7;
-                 #elif defined Py_DEBUG
-                    expected_size = 53;
-                 #else
-                    expected_size = 37;
-                 #endif
-                 if(s->ob_type->tp_basicsize != expected_size)
-                 {
-                     printf("tp_basicsize==%zd\\n", s->ob_type->tp_basicsize); 
-                     result = 0;
-                 }
                  Py_DECREF(s);
-                 return PyBool_FromLong(result);
+                 return PyLong_FromLong(result);
              """),
             ("test_Size_exception", "METH_NOARGS",
              """
                  PyObject* f = PyFloat_FromDouble(1.0);
-                 Py_ssize_t size = PyString_Size(f);
+                 PyBytes_Size(f);
 
                  Py_DECREF(f);
                  return NULL;
              """),
-             ("test_is_string", "METH_VARARGS",
+             ("test_is_bytes", "METH_VARARGS",
              """
-                return PyBool_FromLong(PyString_Check(PyTuple_GetItem(args, 0)));
+                return PyBool_FromLong(PyBytes_Check(PyTuple_GetItem(args, 0)));
              """)], prologue='#include <stdlib.h>')
-        assert module.get_hello1() == 'Hello world'
-        assert module.get_hello2() == 'Hello world'
-        assert module.test_Size()
+        assert module.get_hello1() == b'Hello world'
+        assert module.get_hello2() == b'Hello world'
+        assert module.test_Size() == 11
         raises(TypeError, module.test_Size_exception)
 
-        assert module.test_is_string("")
-        assert not module.test_is_string(())
+        assert module.test_is_bytes(b"")
+        assert not module.test_is_bytes(())
 
-    def test_string_buffer_init(self):
+    def test_bytes_buffer_init(self):
         module = self.import_extension('foo', [
-            ("getstring", "METH_NOARGS",
+            ("getbytes", "METH_NOARGS",
              """
                  PyObject *s, *t;
                  char* c;
-                 Py_ssize_t len;
 
-                 s = PyString_FromStringAndSize(NULL, 4);
+                 s = PyBytes_FromStringAndSize(NULL, 4);
                  if (s == NULL)
                     return NULL;
-                 t = PyString_FromStringAndSize(NULL, 3);
+                 t = PyBytes_FromStringAndSize(NULL, 3);
                  if (t == NULL)
                     return NULL;
                  Py_DECREF(t);
-                 c = PyString_AsString(s);
+                 c = PyBytes_AS_STRING(s);
                  c[0] = 'a';
                  c[1] = 'b';
                  c[2] = 0;
@@ -89,55 +72,63 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
                  return s;
              """),
             ])
-        s = module.getstring()
+        s = module.getbytes()
         assert len(s) == 4
-        assert s == 'ab\x00c'
+        assert s == b'ab\x00c'
 
-    def test_string_tp_alloc(self):
+    def test_bytes_tp_alloc(self):
         module = self.import_extension('foo', [
             ("tpalloc", "METH_NOARGS",
              """
                 PyObject *base;
                 PyTypeObject * type;
-                PyStringObject *obj;
-                char * p_str;
-                base = PyString_FromString("test");
-                if (PyString_GET_SIZE(base) != 4)
-                    return PyLong_FromLong(-PyString_GET_SIZE(base));
+                PyBytesObject *obj;
+                base = PyBytes_FromString("test");
+                if (PyBytes_GET_SIZE(base) != 4)
+                    return PyLong_FromLong(-PyBytes_GET_SIZE(base));
                 type = base->ob_type;
                 if (type->tp_itemsize != 1)
                     return PyLong_FromLong(type->tp_itemsize);
-                obj = (PyStringObject*)type->tp_alloc(type, 10);
-                if (PyString_GET_SIZE(obj) != 10)
-                    return PyLong_FromLong(PyString_GET_SIZE(obj));
+                obj = (PyBytesObject*)type->tp_alloc(type, 10);
+                if (PyBytes_GET_SIZE(obj) != 10)
+                    return PyLong_FromLong(PyBytes_GET_SIZE(obj));
                 /* cannot work, there is only RO access
-                memcpy(PyString_AS_STRING(obj), "works", 6); */
+                memcpy(PyBytes_AS_STRING(obj), "works", 6); */
                 Py_INCREF(obj);
                 return (PyObject*)obj;
              """),
+            ('alloc_rw', "METH_NOARGS",
+             '''
+                PyObject *obj = _PyObject_NewVar(&PyBytes_Type, 10);
+                memcpy(PyBytes_AS_STRING(obj), "works", 6);
+                return (PyObject*)obj;
+             '''),
             ])
+        s = module.alloc_rw()
+        assert s == b'works' + b'\x00' * 5
         s = module.tpalloc()
-        assert s == '\x00' * 10
+        assert s == b'\x00' * 10
 
     def test_AsString(self):
         module = self.import_extension('foo', [
-            ("getstring", "METH_NOARGS",
+            ("getbytes", "METH_NOARGS",
              """
-                 PyObject* s1 = PyString_FromStringAndSize("test", 4);
-                 char* c = PyString_AsString(s1);
-                 PyObject* s2 = PyString_FromStringAndSize(c, 4);
+                 char *c;
+                 PyObject* s2, *s1 = PyBytes_FromStringAndSize("test", 4);
+                 c = PyBytes_AsString(s1);
+                 s2 = PyBytes_FromStringAndSize(c, 4);
                  Py_DECREF(s1);
                  return s2;
              """),
             ])
-        s = module.getstring()
-        assert s == 'test'
+        s = module.getbytes()
+        assert s == b'test'
 
     def test_manipulations(self):
         module = self.import_extension('foo', [
-            ("string_as_string", "METH_VARARGS",
+            ("bytes_as_string", "METH_VARARGS",
              '''
-             return PyString_FromStringAndSize(PyString_AsString(
+             return PyBytes_FromStringAndSize(PyBytes_AsString(
                        PyTuple_GetItem(args, 0)), 4);
              '''
             ),
@@ -147,22 +138,22 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
                 PyObject * left = PyTuple_GetItem(args, 0);
                 Py_INCREF(left);    /* the reference will be stolen! */
                 v = &left;
-                PyString_Concat(v, PyTuple_GetItem(args, 1));
+                PyBytes_Concat(v, PyTuple_GetItem(args, 1));
                 return *v;
              """)])
-        assert module.string_as_string("huheduwe") == "huhe"
-        ret = module.concat('abc', 'def')
-        assert ret == 'abcdef'
+        assert module.bytes_as_string(b"huheduwe") == b"huhe"
+        ret = module.concat(b'abc', b'def')
+        assert ret == b'abcdef'
         ret = module.concat('abc', u'def')
         assert not isinstance(ret, str)
         assert isinstance(ret, unicode)
         assert ret == 'abcdef'
 
-    def test_py_string_as_string_None(self):
+    def test_py_bytes_as_string_None(self):
         module = self.import_extension('foo', [
             ("string_None", "METH_VARARGS",
              '''
-             if (PyString_AsString(Py_None)) {
+             if (PyBytes_AsString(Py_None)) {
                 Py_RETURN_NONE;
              }
              return NULL;
@@ -172,18 +163,18 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
 
     def test_AsStringAndSize(self):
         module = self.import_extension('foo', [
-            ("getstring", "METH_NOARGS",
+            ("getbytes", "METH_NOARGS",
              """
-                 PyObject* s1 = PyString_FromStringAndSize("te\\0st", 5);
+                 PyObject* s1 = PyBytes_FromStringAndSize("te\\0st", 5);
                  char *buf;
                  Py_ssize_t len;
-                 if (PyString_AsStringAndSize(s1, &buf, &len) < 0)
+                 if (PyBytes_AsStringAndSize(s1, &buf, &len) < 0)
                      return NULL;
                  if (len != 5) {
                      PyErr_SetString(PyExc_AssertionError, "Bad Length");
                      return NULL;
                  }
-                 if (PyString_AsStringAndSize(s1, &buf, NULL) >= 0) {
+                 if (PyBytes_AsStringAndSize(s1, &buf, NULL) >= 0) {
                      PyErr_SetString(PyExc_AssertionError, "Should Have failed");
                      return NULL;
                  }
@@ -193,7 +184,7 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
                  return Py_None;
              """),
             ])
-        module.getstring()
+        module.getbytes()
 
     def test_py_string_as_string_Unicode(self):
         module = self.import_extension('foo', [
@@ -276,7 +267,7 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
                  PyObject *s = args;
                  Py_INCREF(s);
                  PyString_InternInPlace(&s);
-                 if (((PyStringObject*)s)->ob_sstate == SSTATE_NOT_INTERNED)
+                 if (((PyBytesObject*)s)->ob_sstate == SSTATE_NOT_INTERNED)
                  {
                     Py_DECREF(s);
                     s = PyString_FromString("interned error");
@@ -294,7 +285,7 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
              ("test_macro_invocations", "METH_NOARGS",
              """
                 PyObject* o = PyString_FromString("");
-                PyStringObject* u = (PyStringObject*)o;
+                PyBytesObject* u = (PyBytesObject*)o;
 
                 PyString_GET_SIZE(u);
                 PyString_GET_SIZE(o);
@@ -311,18 +302,18 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
             ("test_hash", "METH_VARARGS",
              '''
                 PyObject* obj = (PyTuple_GetItem(args, 0));
-                long hash = ((PyStringObject*)obj)->ob_shash;
-                return PyLong_FromLong(hash);  
+                long hash = ((PyBytesObject*)obj)->ob_shash;
+                return PyLong_FromLong(hash);
              '''
              ),
             ("test_sstate", "METH_NOARGS",
              '''
                 PyObject *s = PyString_FromString("xyz");
-                int sstate = ((PyStringObject*)s)->ob_sstate;
-                /*printf("sstate now %d\\n", sstate);*/
+                /*int sstate = ((PyBytesObject*)s)->ob_sstate;
+                printf("sstate now %d\\n", sstate);*/
                 PyString_InternInPlace(&s);
-                sstate = ((PyStringObject*)s)->ob_sstate;
-                /*printf("sstate now %d\\n", sstate);*/
+                /*sstate = ((PyBytesObject*)s)->ob_sstate;
+                printf("sstate now %d\\n", sstate);*/
                 Py_DECREF(s);
                 return PyBool_FromLong(1);
              '''),
@@ -332,27 +323,124 @@ class AppTestStringObject(AppTestCpythonExtensionBase):
         # doesn't really test, but if printf is enabled will prove sstate
         assert module.test_sstate()
 
+    def test_subclass(self):
+        # taken from PyStringArrType_Type in numpy's scalartypes.c.src
+        module = self.import_extension('bar', [
+            ("newsubstr", "METH_O",
+             """
+                PyObject * obj;
+                char * data;
+                int len;
+                PyType_Ready(&PyStringArrType_Type);
 
-class TestString(BaseApiTest):
-    def test_string_resize(self, space, api):
+                data = PyString_AS_STRING(args);
+                len = PyString_GET_SIZE(args);
+                if (data == NULL || len < 1)
+                    Py_RETURN_NONE;
+                obj = PyArray_Scalar(data, len);
+                return obj;
+             """),
+            ], prologue="""
+                #include <Python.h>
+                PyTypeObject PyStringArrType_Type = {
+                    PyObject_HEAD_INIT(NULL)
+                    0,                            /* ob_size */
+                    "bar.string_",                /* tp_name*/
+                    sizeof(PyBytesObject), /* tp_basicsize*/
+                    0                             /* tp_itemsize */
+                    };
+
+                    static PyObject *
+                    stringtype_repr(PyObject *self)
+                    {
+                        const char *dptr, *ip;
+                        int len;
+                        PyObject *new;
+
+                        ip = dptr = PyString_AS_STRING(self);
+                        len = PyString_GET_SIZE(self);
+                        dptr += len-1;
+                        while(len > 0 && *dptr-- == 0) {
+                            len--;
+                        }
+                        new = PyString_FromStringAndSize(ip, len);
+                        if (new == NULL) {
+                            return PyString_FromString("");
+                        }
+                        return new;
+                    }
+
+                    static PyObject *
+                    stringtype_str(PyObject *self)
+                    {
+                        const char *dptr, *ip;
+                        int len;
+                        PyObject *new;
+
+                        ip = dptr = PyString_AS_STRING(self);
+                        len = PyString_GET_SIZE(self);
+                        dptr += len-1;
+                        while(len > 0 && *dptr-- == 0) {
+                            len--;
+                        }
+                        new = PyString_FromStringAndSize(ip, len);
+                        if (new == NULL) {
+                            return PyString_FromString("");
+                        }
+                        return new;
+                    }
+
+                    PyObject *
+                    PyArray_Scalar(char *data, int n)
+                    {
+                        PyTypeObject *type = &PyStringArrType_Type;
+                        PyObject *obj;
+                        void *destptr;
+                        int itemsize = n;
+                        obj = type->tp_alloc(type, itemsize);
+                        if (obj == NULL) {
+                            return NULL;
+                        }
+                        destptr = PyString_AS_STRING(obj);
+                        ((PyBytesObject *)obj)->ob_shash = -1;
+                        memcpy(destptr, data, itemsize);
+                        return obj;
+                    }
+            """, more_init = '''
+                PyStringArrType_Type.tp_alloc = NULL;
+                PyStringArrType_Type.tp_free = NULL;
+
+                PyStringArrType_Type.tp_repr = stringtype_repr;
+                PyStringArrType_Type.tp_str = stringtype_str;
+                PyStringArrType_Type.tp_flags = Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE;
+                PyStringArrType_Type.tp_itemsize = sizeof(char);
+                PyStringArrType_Type.tp_base = &PyString_Type;
+            ''')
+
+        a = module.newsubstr('abc')
+        assert type(a).__name__ == 'string_'
+        assert a == 'abc'
+
+class TestBytes(BaseApiTest):
+    def test_bytes_resize(self, space, api):
         py_str = new_empty_str(space, 10)
         ar = lltype.malloc(PyObjectP.TO, 1, flavor='raw')
-        py_str.c_buffer[0] = 'a'
-        py_str.c_buffer[1] = 'b'
-        py_str.c_buffer[2] = 'c'
+        py_str.c_ob_sval[0] = 'a'
+        py_str.c_ob_sval[1] = 'b'
+        py_str.c_ob_sval[2] = 'c'
         ar[0] = rffi.cast(PyObject, py_str)
         api._PyString_Resize(ar, 3)
-        py_str = rffi.cast(PyStringObject, ar[0])
+        py_str = rffi.cast(PyBytesObject, ar[0])
         assert py_str.c_ob_size == 3
-        assert py_str.c_buffer[1] == 'b'
-        assert py_str.c_buffer[3] == '\x00'
+        assert py_str.c_ob_sval[1] == 'b'
+        assert py_str.c_ob_sval[3] == '\x00'
         # the same for growing
         ar[0] = rffi.cast(PyObject, py_str)
         api._PyString_Resize(ar, 10)
-        py_str = rffi.cast(PyStringObject, ar[0])
+        py_str = rffi.cast(PyBytesObject, ar[0])
         assert py_str.c_ob_size == 10
-        assert py_str.c_buffer[1] == 'b'
-        assert py_str.c_buffer[10] == '\x00'
+        assert py_str.c_ob_sval[1] == 'b'
+        assert py_str.c_ob_sval[10] == '\x00'
         Py_DecRef(space, ar[0])
         lltype.free(ar, flavor='raw')
 
